package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.common.base.RankItem;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.FeatureService;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.service.rank.RankResult;
import com.tzld.piaoquan.recommend.server.service.rank.RankService;
import com.tzld.piaoquan.recommend.server.service.rank.extractor.ExtractorUtils;
import com.tzld.piaoquan.recommend.server.service.rank.extractor.RankExtractorItemTags;
import com.tzld.piaoquan.recommend.server.service.rank.processor.RankProcessorBoost;
import com.tzld.piaoquan.recommend.server.service.rank.processor.RankProcessorDensity;
import com.tzld.piaoquan.recommend.server.service.rank.processor.RankProcessorInsert;
import com.tzld.piaoquan.recommend.server.service.rank.processor.RankProcessorTagFilter;
import com.tzld.piaoquan.recommend.server.service.recall.strategy.*;
import com.tzld.piaoquan.recommend.server.service.score.ScorerUtils;
import com.tzld.piaoquan.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class RankStrategy4RegionMergeModelV999 extends RankStrategy4RegionMergeModelBasic {
    @ApolloJsonValue("${rank.score.merge.weightv567:}")
    private Map<String, Double> mergeWeight;

    @Autowired
    private FeatureService featureService;


    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        Map<String, Double> mergeWeight = this.mergeWeight != null ? this.mergeWeight : new HashMap<>(0);
        //-------------------融-------------------
        //-------------------合-------------------
        //-------------------逻-------------------
        //-------------------辑-------------------

        List<Video> oldRovs = new ArrayList<>();
        oldRovs.addAll(extractAndSort(param, RegionHRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, RegionHDupRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, Region24HRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, RegionRelative24HRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, RegionRelative24HDupRecallStrategy.PUSH_FORM));
        removeDuplicate(oldRovs);
        int sizeReturn = param.getSize();
        List<Video> v0 = oldRovs.size() <= sizeReturn
                ? oldRovs
                : oldRovs.subList(0, sizeReturn);
        Set<Long> setVideo = new HashSet<>();
        this.duplicate(setVideo, v0);


        //-------------------相关性召回 融合+去重-------------------
        List<Video> v5 = extractAndSort(param, SimHotVideoRecallStrategy.PUSH_FORM);
        List<Video> v6 = extractAndSort(param, ReturnVideoRecallStrategy.PUSH_FORM);
        this.duplicate(setVideo, v5);
        this.duplicate(setVideo, v6);
        //-------------------流量池直接送 融合+去重-------------------
        List<Video> v9 = extractAndSort(param, FlowPoolLastDayTopRecallStrategy.PUSH_FORM);
        this.duplicate(setVideo, v9);
        //-------------------地域相关召回 融合+去重-------------------
        List<Video> v1 = extractAndSort(param, RegionRealtimeRecallStrategyV1.PUSH_FORM);
        this.duplicate(setVideo, v1);
        //-------------------节日扶持召回 融合+去重-------------------
        List<Video> v7 = extractAndSort(param, FestivalRecallStrategyV1.PUSH_FORM);
        this.duplicate(setVideo, v7);
        List<Video> rovRecallRank = new ArrayList<>();
        rovRecallRank.addAll(v0);
        rovRecallRank.addAll(v5.subList(0, Math.min(mergeWeight.getOrDefault("v5", 5.0).intValue(), v5.size())));
        rovRecallRank.addAll(v6.subList(0, Math.min(mergeWeight.getOrDefault("v6", 5.0).intValue(), v6.size())));
        rovRecallRank.addAll(v9.subList(0, Math.min(mergeWeight.getOrDefault("v9", 5.0).intValue(), v9.size())));
        rovRecallRank.addAll(v1.subList(0, Math.min(mergeWeight.getOrDefault("v1", 5.0).intValue(), v1.size())));
        rovRecallRank.addAll(v7.subList(0, Math.min(mergeWeight.getOrDefault("v7", 5.0).intValue(), v7.size())));

        //-------------------排-------------------
        //-------------------序-------------------
        //-------------------逻-------------------
        //-------------------辑-------------------

        // TODO 1 批量获取特征  省份参数要对齐
        List<String> vids = CommonCollectionUtils.toListDistinct(rovRecallRank, v -> String.valueOf(v.getVideoId()));
        // k1:视频、k2:表、k3:特征、v:特征值
        Map<String, Map<String, Map<String, String>>> featureMap = featureService.getFeature(vids,
                String.valueOf(param.getAppType()), param.getProvince());

        // TODO 2 特征处理
        List<RankItem> rankItems = CommonCollectionUtils.toList(rovRecallRank, RankItem::new);
        Map<String, Map<String, String>> videoFeatureMap = extractVideoFeature(featureMap);
        for (RankItem item : rankItems) {
            item.setFeatureMap(videoFeatureMap.get(item.getVideoId() + ""));
        }
        Map<String, String> sceneFeatureMap = extractSceneFeature(featureMap);
        Map<String, String> userFeatureMap = extractUserFeature(featureMap);

        // TODO 3 排序
        List<RankItem> rovRecallScore = ScorerUtils.getScorerPipeline(ScorerUtils.BASE_CONF)
                .scoring(sceneFeatureMap, userFeatureMap, rankItems);

        // TODO
        List<Video> result = CommonCollectionUtils.toList(rovRecallScore, RankItem::getVideo);

        return result;
    }

    private Map<String, Map<String, String>> extractVideoFeature(Map<String, Map<String, Map<String, String>>> featureMap) {
        // TODO
        return null;
    }

    private Map<String, String> extractSceneFeature(Map<String, Map<String, Map<String, String>>> featureMap) {
        // TODO
        return null;
    }

    private Map<String, String> extractUserFeature(Map<String, Map<String, Map<String, String>>> featureMap) {
        // TODO
        return null;
    }


}
